/*===============================================================================================
 programmer_sound Example
 Copyright (c), Firelight Technologies Pty, Ltd 2006-2007.

 Demonstrates how to use the "programmer sound" and "programmer selected sound definition"
 features of the FMOD event system
===============================================================================================*/
#include "../../api/inc/fmod_event.hpp"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

const char   *MEDIA_PATH = "..\\media\\";
const char   *FSB_NAME   = "..\\media\\other.fsb";

static int g_subsound_index = 0;
static int g_sounddef_entry_index = 0;

FMOD::Sound  *fsb;
FMOD::System *sys;


void ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
        printf("FMOD error! (%d)\n", result);
        exit(-1);
    }
}


FMOD_RESULT F_CALLBACK eventcallback(FMOD_EVENT *event, FMOD_EVENT_CALLBACKTYPE type, void *param1, void *param2, void *userdata)
{
    FMOD_RESULT result;

    switch (type)
    {
        case FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_CREATE :
        {
            char *name      = (char *)param1;               // [in]  (char *) name of sound definition
            int entryindex  = *((int *)param2);             // [in]  (int) index of sound definition entry
            FMOD::Sound **s = (FMOD::Sound **)param2;       // [out] (FMOD::Sound *) a valid lower level API FMOD Sound handle
            char sound_name[128];

            result = fsb->getSubSound(g_subsound_index, s);
            ERRCHECK(result);
            result = (*s)->getName(sound_name, 128);
            ERRCHECK(result);
            printf("FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_CREATE '%s' (%d)\n",
                   sound_name, g_subsound_index);
            break;
        }

        case FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_RELEASE :
        {
            FMOD::Sound *s = (FMOD::Sound *)param2;         // [in]  (FMOD::Sound *) the FMOD sound handle that was previously created in FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_CREATE
            char sound_name[128];

            result = s->getName(sound_name, 128);
            ERRCHECK(result);
            printf("FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_RELEASE '%s'\n", sound_name);
            break;
        }

        case FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_SELECTINDEX :
        {
            char *name = (char *)param1;                    // [in]  (char *) name of sound definition
            int *index = (int *)param2;                     // [out] (int *) the sounddef entry index to use

            *index = g_sounddef_entry_index < *index ? g_sounddef_entry_index
                                                     : *index - 1;

            printf("FMOD_EVENT_CALLBACKTYPE_SOUNDDEF_SELECTINDEX '%s': %d\n",
                   name, *index);
            break;
        }
    }

    return FMOD_OK;
}


int main(int argc, char *argv[])
{
    FMOD_RESULT        result;
    FMOD::EventSystem *eventsystem;
    FMOD::EventGroup  *eventgroup;
    FMOD::Event       *event;
    int                key;

    printf("======================================================================\n");
    printf("Programmer Sound. Copyright (c) Firelight Technologies 2006-2007.\n");
    printf("======================================================================\n");

    result = FMOD::EventSystem_Create(&eventsystem);
    ERRCHECK(result);
    result = eventsystem->init(64, FMOD_INIT_NORMAL, 0, FMOD_EVENT_INIT_NORMAL);
    ERRCHECK(result);
    result = eventsystem->setMediaPath((char *)MEDIA_PATH);
    ERRCHECK(result);
    result = eventsystem->load("examples.fev", 0, 0);
    ERRCHECK(result);
    result = eventsystem->getGroup("examples/examples", FMOD_EVENT_DEFAULT, &eventgroup);
    ERRCHECK(result);

    result = eventsystem->getSystemObject(&sys);
    ERRCHECK(result);
    result = sys->createStream(FSB_NAME, FMOD_2D | FMOD_SOFTWARE, 0, &fsb);
    ERRCHECK(result);

    printf("======================================================================\n");
    printf("Press '1' to start a 'Programmer Sound' event\n");
    printf("Press '2' to start a 'Programmer Selected' event\n");
    printf("Press '+' or '=' to increase subsound index\n");
    printf("Press '-' or '_' to decrease subsound index\n");
    printf("Press '>' or '.' to increase sound definition entry index\n");
    printf("Press '<' or ',' to decrease sound definition entry index\n");
    printf("Press 'Esc' to quit\n");
    printf("======================================================================\n");
    printf("Subsound index = %d\n", g_subsound_index);
    printf("Sound definition entry index = %d\n", g_sounddef_entry_index);

    key = 0;
    do
    {
        if (_kbhit())
        {
            key = _getch();

            switch(key)
            {
            case '1':
                result = eventgroup->getEvent("programmersound", FMOD_EVENT_DEFAULT, &event);
                if (result == FMOD_OK)
                {
                    result = event->setCallback(eventcallback, 0);
                    ERRCHECK(result);
                    result = event->start();
                    ERRCHECK(result);
                }
                break;
            case '2':
                result = eventgroup->getEvent("programmerselected", FMOD_EVENT_DEFAULT, &event);
                if (result == FMOD_OK)
                {
                    result = event->setCallback(eventcallback, 0);
                    ERRCHECK(result);
                    result = event->start();
                    ERRCHECK(result);
                }
                break;
            case '+': case '=':
                ++g_subsound_index;
                g_subsound_index = g_subsound_index > 3 ? 3 : g_subsound_index;
                printf("Subsound index = %d\n", g_subsound_index);
                break;
            case '-': case '_':
                --g_subsound_index;
                g_subsound_index = g_subsound_index < 0 ? 0 : g_subsound_index;
                printf("Subsound index = %d\n", g_subsound_index);
                break;
            case '>': case '.':
                ++g_sounddef_entry_index;
                printf("Sound definition entry index = %d\n", g_sounddef_entry_index);
                break;
            case '<': case ',':
                --g_sounddef_entry_index;
                g_sounddef_entry_index = g_sounddef_entry_index < 0 ?
                                                0 : g_sounddef_entry_index;
                printf("Sound definition entry index = %d\n", g_sounddef_entry_index);
                break;
            }
        }

        result = eventsystem->update();
        ERRCHECK(result);

        Sleep(10);

    } while (key != 27);

    result = eventsystem->unload();
    ERRCHECK(result);
    result = fsb->release();
    ERRCHECK(result);
    result = eventsystem->release();
    ERRCHECK(result);

    return 0;
}
